@@ -1,7 +1,7 @@
 /* This goofed-up box whacked into shape by Elliot Lee <sopwith@cuc.edu>
    (from the original listbox by Erik Troan <ewt@redhat.com>)
    and contributed to newt for use under the LGPL license.
-   Copyright Elliot Lee 1996 */
+   Copyright (C) 1996, 1997 Elliot Lee */
 
 #include <slang/slang.h>
 #include <stdio.h>
@@ -15,13 +15,14 @@
 /* Linked list of items in the listbox */
 struct items {
     void *key, *data;
+    unsigned char isSelected;
     struct items *next;
 };
 
 /* Holds all the relevant information for this listbox */
 struct listbox {
     newtComponent sb; /* Scrollbar on right side of listbox */
-    int numItems, curWidth;
+    int numItems, curWidth, numSelected;
     int userHasSetWidth;
     int currItem, startShowItem; /* startShowItem is the first item displayed
 				   on the screen */
@@ -37,6 +38,7 @@ struct listbox {
 static void listboxDraw(newtComponent co);
 static void listboxDestroy(newtComponent co);
 static struct eventResult listboxEvent(newtComponent co, struct event ev);
+static void newtListboxRealSetCurrent(newtComponent co);
 
 static struct componentOps listboxOps = {
     listboxDraw,
@@ -59,10 +61,11 @@ newtComponent newtListbox(int left, int top, int height, int flags) {
     li->boxItems = NULL;
     li->numItems = 0;
     li->currItem = 0;
+    li->numSelected = 0;
     li->isActive = 0;
     li->userHasSetWidth = 0;
     li->startShowItem = 0;
-    li->flags = flags & (NEWT_FLAG_RETURNEXIT|NEWT_FLAG_DOBORDER);
+    li->flags = flags & (NEWT_FLAG_RETURNEXIT|NEWT_FLAG_DOBORDER|NEWT_FLAG_MULTIPLE);
 
     if (height) {
 	li->grow = 0;
@@ -84,11 +87,13 @@ newtComponent newtListbox(int left, int top, int height, int flags) {
     li->curWidth = 5;
     co->ops = &listboxOps;
     co->takesFocus = 1;
+    co->callback = NULL;
 
     return co;
 }
 
-void newtListboxSetCurrent(newtComponent co, int num) {
+void newtListboxSetCurrent(newtComponent co, int num)
+{
     struct listbox * li = co->data;
     if (num >= li->numItems)
 	li->currItem = li->numItems - 1;
@@ -105,12 +110,19 @@ void newtListboxSetCurrent(newtComponent co, int num) {
 	li->startShowItem = li->numItems - co->height;
     if(li->startShowItem < 0)
 	li->startShowItem = 0;
+    newtListboxRealSetCurrent(co);
+}
+
+static void
+newtListboxRealSetCurrent(newtComponent co)
+{
+    struct listbox * li = co->data;
     if(li->sb)
 	newtScrollbarSet(li->sb, li->currItem + 1, li->numItems);
     listboxDraw(co);
+    if(co->callback) co->callback(co, co->callbackData);
 }
 
-
 void newtListboxSetWidth(newtComponent co , int width) {
     struct listbox * li = co->data;
     
@@ -134,6 +146,67 @@ void * newtListboxGetCurrent(newtComponent co) {
 	return NULL;
 }
 
+void newtListboxSelectItem(newtComponent co, int item,
+	enum newtFlagsSense sense)
+{
+    struct listbox * li = co->data;
+    int i;
+    struct items *iitem;
+    
+    for(i = 0, iitem = li->boxItems; iitem != NULL && i < item;
+	i++, iitem = iitem->next);
+
+    if (iitem) {
+	if(iitem->isSelected && sense != NEWT_FLAGS_SET)
+	    li->numSelected--;
+	else if(!iitem->isSelected && sense != NEWT_FLAGS_RESET)
+	    li->numSelected++;
+	switch(sense) {
+		case NEWT_FLAGS_RESET:
+			iitem->isSelected = 0; break;
+		case NEWT_FLAGS_SET:
+			iitem->isSelected = 1; break;
+		case NEWT_FLAGS_TOGGLE:
+			iitem->isSelected = !iitem->isSelected;
+	}
+    }
+    listboxDraw(co);
+}
+
+void newtListboxClearSelection(newtComponent co)
+{
+    struct items *item;
+    struct listbox * li = co->data;
+
+    for(item = li->boxItems; item != NULL;
+	item = item->next)
+	item->isSelected = 0;
+    li->numSelected = 0;
+    listboxDraw(co);
+}
+
+/* Free the returned array after use, but NOT the values in the array */
+void ** newtListboxGetSelection(newtComponent co, int *numitems)
+{
+    struct listbox * li;
+    int i;
+    void **retval;
+    struct items *item;
+
+    if(!co || !numitems) return NULL;
+
+    li = co->data;
+    if(!li || !li->numSelected) return NULL;
+
+    retval = malloc(li->numSelected * sizeof(void *));
+    for(i = 0, item = li->boxItems; item != NULL;
+	item = item->next)
+	if(item->isSelected)
+	    retval[i++] = item->data;
+    *numitems = li->numSelected;
+    return retval;
+}
+
 void newtListboxSetText(newtComponent co, int num, char * text) {
     struct listbox * li = co->data;
     int i;
@@ -149,7 +222,7 @@ void newtListboxSetText(newtComponent co, int num, char * text) {
 	item->key = strdup(text);
     }
     if (li->userHasSetWidth == 0
-	&& strlen(text) > li->curWidth) {
+	&& strlen(text) > (size_t)li->curWidth) {
 	co->width = li->curWidth = strlen(text);
 	if (li->sb)
 	    li->sb->left = co->left + co->width + 2;
@@ -187,11 +260,12 @@ int newtListboxAddEntry(newtComponent co, char * text, void * data) {
     }
 
     if (li->userHasSetWidth == 0
-	&& text && (strlen(text) > li->curWidth))
+	&& text && (strlen(text) > (size_t)li->curWidth))
 	li->curWidth = strlen(text) ;
 
     item->key = strdup(text); item->data = data; item->next = NULL;
-
+    item->isSelected = 0;
+    
     if (li->sb)
 	li->sb->left = co->left + li->curWidth + 2;
 
@@ -214,7 +288,7 @@ int newtListboxInsertEntry(newtComponent co, char * text, void * data,
 	num = li->numItems;
 
     if (li->boxItems) {
-	if(num > 1) {
+	if(num > 0) {
 	    for(i = 0, item = li->boxItems; item->next != NULL && i < num - 1;
 		item = item->next, i++);
 	    t = item->next;
@@ -231,16 +305,18 @@ int newtListboxInsertEntry(newtComponent co, char * text, void * data,
     }
 
     if (li->userHasSetWidth == 0
-	&& text && (strlen(text) > li->curWidth))
+	&& text && (strlen(text) > (size_t)li->curWidth))
 	li->curWidth = strlen(text);
 
     item->key = strdup(text?text:"(null)"); item->data = data;
-
+    item->isSelected = 0;
+    
     if (li->sb)
 	li->sb->left = co->left + li->curWidth + 2;
     if (li->userHasSetWidth == 0)
 	co->width = li->curWidth;
     li->numItems++;
+
     listboxDraw(co);
 
     return li->numItems;
@@ -249,7 +325,7 @@ int newtListboxInsertEntry(newtComponent co, char * text, void * data,
 int newtListboxDeleteEntry(newtComponent co, int num) {
     struct listbox * li = co->data;
     int i, widest = 0, t;
-    struct items *item, *item2;
+    struct items *item, *item2 = NULL;
 
     if(num > li->numItems)
 	num = li->numItems;
@@ -296,6 +372,23 @@ int newtListboxDeleteEntry(newtComponent co, int num) {
     return li->numItems;
 }
 
+void newtListboxClear(newtComponent co)
+{
+    struct listbox * li;
+    struct items *anitem, *nextitem;
+    if(co == NULL || (li = co->data) == NULL)
+	return;
+    for(anitem = li->boxItems; anitem != NULL; anitem = nextitem) {
+	nextitem = anitem->next;
+	free(anitem->key);
+	free(anitem);
+    }
+    li->numItems = li->numSelected = li->currItem = li->startShowItem = 0;
+    li->boxItems = NULL;
+    if(li->userHasSetWidth == 0)
+	li->curWidth = 0;
+}
+
 /* If you don't want to get back the text, pass in NULL for the ptr-ptr. Same
    goes for the data. */
 void newtListboxGetEntry(newtComponent co, int num, char **text, void **data) {
@@ -354,13 +447,18 @@ static void listboxDraw(newtComponent co)
 	if (!item->key) continue;
 
 	newtGotorc(co->top + i, co->left + 1);
-	if(j + i == li->currItem)
-	    SLsmg_set_color(NEWT_COLORSET_ACTLISTBOX);
-
+	if(j + i == li->currItem) {
+	    if(item->isSelected)
+		SLsmg_set_color(NEWT_COLORSET_ACTSELLISTBOX);
+	    else
+		SLsmg_set_color(NEWT_COLORSET_ACTLISTBOX);
+	} else if(item->isSelected)
+	    SLsmg_set_color(NEWT_COLORSET_SELLISTBOX);
+	else
+	    SLsmg_set_color(NEWT_COLORSET_LISTBOX);
+	    
 	SLsmg_write_nstring(item->key, li->curWidth);
 
-	if(j + i == li->currItem)
-	    SLsmg_set_color(NEWT_COLORSET_LISTBOX);
     }
     newtGotorc(co->top + (li->currItem - li->startShowItem), co->left);
 }
@@ -380,24 +478,15 @@ static struct eventResult listboxEvent(newtComponent co, struct event ev) {
 	if (!li->isActive) break;
 
 	switch(ev.u.key) {
-	  case NEWT_KEY_ENTER:
-	    if(li-> flags & NEWT_FLAG_RETURNEXIT)
-		er.result = ER_EXITFORM;
-	    break;
-
-	  case NEWT_KEY_UP:
-	    if(li->currItem > 0) {
-		li->currItem--;
-		if(li->currItem < li->startShowItem)
-		    li->startShowItem = li->currItem;
-		if(li->sb)
-		    newtScrollbarSet(li->sb, li->currItem + 1, li->numItems);
-		listboxDraw(co);
-	    }
+	  case ' ':
+	    if(!(li->flags & NEWT_FLAG_MULTIPLE)) break;
+	    newtListboxSelectItem(co, li->currItem, NEWT_FLAGS_TOGGLE);
 	    er.result = ER_SWALLOWED;
-	    break;
+	    /* We don't break here, because it is cool to be able to
+	       hold space to select a bunch of items in a list at once */
 
 	  case NEWT_KEY_DOWN:
+	    if(li->numItems <= 0) break;
 	    if(li->currItem < li->numItems - 1) {
 		li->currItem++;
 		if(li->currItem > (li->startShowItem + co->height - 1)) {
@@ -409,23 +498,64 @@ static struct eventResult listboxEvent(newtComponent co, struct event ev) {
 		    newtScrollbarSet(li->sb, li->currItem + 1, li->numItems);
 		listboxDraw(co);
 	    }
+	    if(co->callback) co->callback(co, co->callbackData);
+	    er.result = ER_SWALLOWED;
+	    break;
+
+	  case NEWT_KEY_ENTER:
+	    if(li->numItems <= 0) break;
+	    if(li->flags & NEWT_FLAG_RETURNEXIT)
+		er.result = ER_EXITFORM;
+	    break;
+
+	  case NEWT_KEY_UP:
+	    if(li->numItems <= 0) break;
+	    if(li->currItem > 0) {
+		li->currItem--;
+		if(li->currItem < li->startShowItem)
+		    li->startShowItem = li->currItem;
+		if(li->sb)
+		    newtScrollbarSet(li->sb, li->currItem + 1, li->numItems);
+		listboxDraw(co);
+	    }
+	    if(co->callback) co->callback(co, co->callbackData);
 	    er.result = ER_SWALLOWED;
 	    break;
 
 	  case NEWT_KEY_PGUP:
-	    newtListboxSetCurrent(co, li->currItem - co->height + 1);
+	    if(li->numItems <= 0) break;
+	    li->startShowItem -= co->height - 1;
+	    if(li->startShowItem < 0)
+		li->startShowItem = 0;
+	    li->currItem -= co->height - 1;
+	    if(li->currItem < 0)
+		li->currItem = 0;
+	    newtListboxRealSetCurrent(co);
 	    er.result = ER_SWALLOWED;
 	    break;
 
 	  case NEWT_KEY_PGDN:
-	    newtListboxSetCurrent(co, li->currItem + co->height - 1);
+	    if(li->numItems <= 0) break;
+	    li->startShowItem += co->height;
+	    if(li->startShowItem > (li->numItems - co->height)) {
+		li->startShowItem = li->numItems - co->height;
+	    }
+	    li->currItem += co->height;
+	    if(li->currItem > li->numItems) {
+		li->currItem = li->numItems - 1;
+	    }
+	    newtListboxRealSetCurrent(co);
 	    er.result = ER_SWALLOWED;
 	    break;
+
 	  case NEWT_KEY_HOME:
+	    if(li->numItems <= 0) break;
 	    newtListboxSetCurrent(co, 0);
 	    er.result = ER_SWALLOWED;
 	    break;
+
 	  case NEWT_KEY_END:
+	    if(li->numItems <= 0) break;
 	    newtListboxSetCurrent(co, li->numItems - 1);
 	    er.result = ER_SWALLOWED;
 	    break;
