@@ -14,6 +14,7 @@
 struct Window {
     int height, width, top, left;
     short * buffer;
+    char * title;
 };
 
 struct keymap {
@@ -56,6 +57,8 @@ struct newtColors newtDefaultColorPalette = {
 	"red",					/* scale empty */
 	"blue", "lightgray",			/* disabled entry fg, bg */
 	"white", "blue",			/* compact button fg, bg */
+	"yellow", "red",			/* active & sel listbox */
+	"black", "brown"			/* selected listbox */
 };
 
 static struct keymap keymap[] = {
@@ -221,6 +224,11 @@ void newtSetColors(struct newtColors colors) {
 
     SLtt_set_color(NEWT_COLORSET_COMPACTBUTTON, "", colors.compactButtonFg,
 			colors.compactButtonBg);
+    
+    SLtt_set_color(NEWT_COLORSET_ACTSELLISTBOX, "", colors.actSelListboxFg,
+		   colors.actSelListboxBg);
+    SLtt_set_color(NEWT_COLORSET_SELLISTBOX, "", colors.selListboxFg,
+		   colors.selListboxBg);
 }
 
 int newtGetKey(void) {
@@ -303,8 +311,9 @@ void newtClearKeyBuffer(void) {
 
 int newtOpenWindow(int left, int top, int width, int height, 
 			  char * title) {
-    int i, j, row, col;
+    int j, row, col;
     int n;
+    int i;
 
     newtFlushInput();
 
@@ -318,6 +327,7 @@ int newtOpenWindow(int left, int top, int width, int height,
     currentWindow->top = top;
     currentWindow->width = width;
     currentWindow->height = height;
+    currentWindow->title = title ? strdup(title) : NULL;
 
     currentWindow->buffer = malloc(sizeof(short) * (width + 3) * (height + 3));
 
@@ -334,8 +344,8 @@ int newtOpenWindow(int left, int top, int width, int height,
     SLsmg_set_color(NEWT_COLORSET_BORDER);
     SLsmg_draw_box(top - 1, left - 1, height + 2, width + 2);
 
-    if (title) {
-	i = strlen(title) + 4;
+    if (currentWindow->title) {
+	i = strlen(currentWindow->title) + 4;
 	i = ((width - i) / 2) + left;
 	SLsmg_gotorc(top - 1, i);
 	SLsmg_set_char_set(1);
@@ -343,7 +353,7 @@ int newtOpenWindow(int left, int top, int width, int height,
 	SLsmg_set_char_set(0);
 	SLsmg_write_char(' ');
 	SLsmg_set_color(NEWT_COLORSET_TITLE);
-	SLsmg_write_string(title);
+	SLsmg_write_string(currentWindow->title);
 	SLsmg_set_color(NEWT_COLORSET_BORDER);
 	SLsmg_write_char(' ');
 	SLsmg_set_char_set(1);
@@ -382,6 +392,7 @@ void newtPopWindow(void) {
     }
 
     free(currentWindow->buffer);
+    free(currentWindow->title);
 
     if (currentWindow == windowStack) 
 	currentWindow = NULL;
@@ -539,7 +550,15 @@ int newtSetFlags(int oldFlags, int newFlags, enum newtFlagsSense sense) {
       case NEWT_FLAGS_RESET:
 	return oldFlags & (~newFlags); 
 
+      case NEWT_FLAGS_TOGGLE:
+	return oldFlags ^ newFlags;
+
       default:
 	return oldFlags;
     }
 }
+
+void newtBell(void)
+{
+	SLtt_beep();
+}
