@@ -26,6 +26,8 @@ extern "C" {
 #define NEWT_COLORSET_FULLSCALE		20
 #define NEWT_COLORSET_DISENTRY		21
 #define NEWT_COLORSET_COMPACTBUTTON	22
+#define NEWT_COLORSET_ACTSELLISTBOX	23
+#define NEWT_COLORSET_SELLISTBOX	24
 
 struct newtColors {
     char * rootFg, * rootBg;
@@ -48,9 +50,11 @@ struct newtColors {
     char * emptyScale, * fullScale;
     char * disabledEntryFg, * disabledEntryBg;
     char * compactButtonFg, * compactButtonBg;
+    char * actSelListboxFg, * actSelListboxBg;
+    char * selListboxFg, * selListboxBg;
 };
 
-enum newtFlagsSense { NEWT_FLAGS_SET, NEWT_FLAGS_RESET };
+enum newtFlagsSense { NEWT_FLAGS_SET, NEWT_FLAGS_RESET, NEWT_FLAGS_TOGGLE };
 
 #define NEWT_FLAG_RETURNEXIT 	(1 << 0)
 #define NEWT_FLAG_HIDDEN 	(1 << 1)
@@ -60,6 +64,8 @@ enum newtFlagsSense { NEWT_FLAGS_SET, NEWT_FLAGS_RESET };
 #define NEWT_FLAG_DOBORDER	(1 << 5)
 #define NEWT_FLAG_WRAP		(1 << 6)
 #define NEWT_FLAG_NOF12		(1 << 7)
+#define NEWT_FLAG_MULTIPLE      (1 << 8)
+#define NEWT_FLAG_SELECTED	(1 << 9)
 
 /* Backwards compatibility */
 #define NEWT_LISTBOX_RETURNEXIT NEWT_FLAG_RETURNEXIT
@@ -98,6 +104,7 @@ void newtPushHelpLine(char * text);
 void newtRedrawHelpLine(void);
 void newtPopHelpLine(void);
 void newtDrawRootText(int row, int col, char * text);
+void newtBell(void);
 
 /* Components */
 
@@ -130,8 +137,15 @@ void newtListboxSetData(newtComponent co, int num, void * data);
 int newtListboxAddEntry(newtComponent co, char * text, void * data);
 int newtListboxInsertEntry(newtComponent co, char * text, void * data, int num);
 int newtListboxDeleteEntry(newtComponent co, int num);
+void newtListboxClear(newtComponent co); /* removes all entries from listbox */
 void newtListboxGetEntry(newtComponent co, int num, char **text, void **data);
+/* Returns an array of data pointers from items, last element is NULL */
+void **newtListboxGetSelection(newtComponent co, int *numitems);
+void newtListboxClearSelection(newtComponent co);
+void newtListboxSelectItem(newtComponent co, int item,
+	enum newtFlagsSense sense);
 
+    
 newtComponent newtTextbox(int left, int top, int with, int height, int flags);
 void newtTextboxSetText(newtComponent co, const char * text);
 void newtTextboxSetHeight(newtComponent co, int height);
@@ -164,7 +178,7 @@ void newtEntrySet(newtComponent co, char * value, int cursorAtEnd);
 void newtEntrySetFlags(newtComponent co, int flags, enum newtFlagsSense sense);
 
 newtComponent newtScale(int left, int top, int width, long long fullValue);
-void newtScaleSet(newtComponent co, long long amount);
+void newtScaleSet(newtComponent co, unsigned long long amount);
 
 void newtComponentAddCallback(newtComponent co, newtCallback f, void * data);
 
